#ifndef UTILS_UNITS_H
#define UTILS_UNITS_H 1

namespace utils
{
    /// Define a simple custom measure
    /// @param M  The measure (unit)
    /// @param T  The dimensions of the measure
    /// @param N  The name/suffix of the measure
#define DECLARE_CUSTOM_UNIT( M, D, N, S )  \
    namespace units { \
    struct M : D {};  \
    template <>  inline string Name< M >() { return (N); }  \
    template <>  inline float64 CustomScale< M >() { return (S); }  \
    } \
    DECLARE_COMMON_UNIT_BASE(M)

    //@{
    /// @name Miscellaneous other custom measures
    DECLARE_CUSTOM_UNIT(  pounds,            mass,        "lb",             453.59237       ) // Avoirdupois
    DECLARE_CUSTOM_UNIT(  pints,             volume,      "pints",          0.56826125      ) // UK liquid
    DECLARE_CUSTOM_UNIT(  gallons,           volume,      "gal",            4.54609         ) // UK
    DECLARE_CUSTOM_UNIT(  usgallons,         volume,      "gal(US)",        3.785411784     ) // liquid - 128 fl .oz. [US]
    DECLARE_CUSTOM_UNIT(  bushels,           volume,      "bushels",        36.36872        ) // 8 imperial gallons
    DECLARE_CUSTOM_UNIT(  mpg,               travelfuel,  "mpg",            1609.3/4.54609  ) // UK
    DECLARE_CUSTOM_UNIT(  inches,            length,      "in",             0.0254          )
    DECLARE_CUSTOM_UNIT(  feet,              length,      "ft",             0.3048          )
    DECLARE_CUSTOM_UNIT(  flightlevel,       length,      "FL",             30.48           ) // Altitude in feet/100 relative to sea level with pressure of 1013.25mB
    DECLARE_CUSTOM_UNIT(  yards,             length,      "yds",            0.3048*3.0      ) // 3 feet, in turn is 0.3048 m.
    DECLARE_CUSTOM_UNIT(  miles,             length,      "mi",             1609.3          )
    DECLARE_CUSTOM_UNIT(  nauticalmiles,     length,      "naut",           1852.0          )
    DECLARE_CUSTOM_UNIT(  hands,             length,      "hands",          0.1016          ) // 4 inches
    DECLARE_CUSTOM_UNIT(  chains,            length,      "chains",         20.1168         ) // 66 feet
    DECLARE_CUSTOM_UNIT(  links,             length,      "links",          20.1168/100.0   ) // 100 links per chain
    DECLARE_CUSTOM_UNIT(  cubits,            length,      "cubits",         0.4572          ) // 1.5 feet
    DECLARE_CUSTOM_UNIT(  rods,              length,      "rods",           5.0292          ) // 16.5 feet / 11 cubits
    DECLARE_CUSTOM_UNIT(  perches,           length,      "perches",        5.0292          ) // ditto
    DECLARE_CUSTOM_UNIT(  poles,             length,      "poles",          5.0292          ) // ditto
    DECLARE_CUSTOM_UNIT(  paces,             length,      "paces",          1.48            ) // 5 Roman feet
    DECLARE_CUSTOM_UNIT(  furlongs,          length,      "furlongs",       201.168         ) // 660 feet (8 per mile)
    DECLARE_CUSTOM_UNIT(  leagues,           length,      "leagues",        5556.0          ) // 3 nm
    DECLARE_CUSTOM_UNIT(  fathoms,           length,      "fathoms",        1.8288          ) // 6 ft
    DECLARE_CUSTOM_UNIT(  angstroms,         length,      "Å",              100.0*1e-12     ) // 0.1 nm
    DECLARE_CUSTOM_UNIT(  lightyears,        length,      "ly",             9460730472580800.0 )
    DECLARE_CUSTOM_UNIT(  au,                length,      "AU",             149597870691.0  ) // Astronomical Unit
    DECLARE_CUSTOM_UNIT(  points,            length,      "pt",             0.0254/72.0     ) // 1/72 inches
    DECLARE_CUSTOM_UNIT(  squarekilometres,  area,        "(km)²",          1000000.0       )
    DECLARE_CUSTOM_UNIT(  hectares,          area,        "he",             10000.0         )
    DECLARE_CUSTOM_UNIT(  acres,             area,        "acres",          4046.8564224    )
    DECLARE_CUSTOM_UNIT(  squaremils,        area,        "square-mils",    6.4516e-10      ) // millionths of a square inch
    DECLARE_CUSTOM_UNIT(  circularmils,      area,        "circular-mils",  5.066e-10       ) // 0.7854 mill. of a sq inch
    DECLARE_CUSTOM_UNIT(  mph,               velocity,    "mph",            1609.3/3600.0   ) // miles per hour
    DECLARE_CUSTOM_UNIT(  kph,               velocity,    "kph",            1000.0/3600.0   ) // km per hour
    DECLARE_CUSTOM_UNIT(  knots,             velocity,    "knots",          1852.0/3600.0   ) // naut per hour
    DECLARE_CUSTOM_UNIT(  minutes,           time,        "min",            60.0            )
    DECLARE_CUSTOM_UNIT(  hours,             time,        "hr",             60.0*60.0       )
    DECLARE_CUSTOM_UNIT(  days,              time,        "days",           24.0*60.0*60.0  )
    DECLARE_CUSTOM_UNIT(  warhols,           time,        "warhols",        15.0*60.0       ) // 15 mins
    DECLARE_CUSTOM_UNIT(  atmosphere,        pressure,    "atm",            101325.0        )
    DECLARE_CUSTOM_UNIT(  psi,               pressure,    "psi",            6894.76         )
    DECLARE_CUSTOM_UNIT(  horsepower,        power,       "hp",             745.69987158227022 ) // 33,000 ft·lbf/min
    DECLARE_CUSTOM_UNIT(  abstract,          scalar,      "",               1.0 ) // conversion is meaningless - it's abstract
    //@}

} // namespace utils

#endif /* UTILS_UNITS_H */
